#!/bin/sh
cd /usr
tar zcvf /dist/shutl-1.6Bsrc.tar.Z src/shellutils-1.6
tar zcvf /dist/shutl-1.6Bsbin.tar.Z src/shellutils-1.6/C* \
src/shellutils-1.6/R* \
bin/[ bin/basename bin/date bin/dirname bin/env bin/expr \
bin/groups bin/id bin/logname bin/nice bin/nohup \
bin/pathchk bin/printenv bin/printf bin/sleep bin/tee bin/test bin/tty \
bin/uname bin/whoami bin/yes \
man/man1/basename.1 man/man1/date.1 man/man1/dirname.1 \
man/man1/env.1 man/man1/expr.1 man/man1/groups.1 man/man1/id.1 \
man/man1/logname.1 man/man1/nice.1 man/man1/nohup.1 \
man/man1/pathchk.1 man/man1/printenv.1 man/man1/printf.1 \
man/man1/sleep.1 man/man1/tee.1 man/man1/test.1 man/man1/tty.1 \
man/man1/uname.1 man/man1/whoami.1 man/man1/yes.1
